import enum
from pydoc import locate  # https://stackoverflow.com/a/29831586
from re import search


@enum.unique
class Status(enum.Enum):
    ACTIVE = "ACTIVE"
    SKIPPED = "SKIPPED"
    PASSED = "PASSED"
    FAILED = "FAILED"


@enum.unique
class Method(enum.Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"


@enum.unique
class Comparison(enum.Enum):
    # equality

    EQUALS = "equals"

    def is_equals(self, actual, expect):
        return actual == expect

    NOT_EQUALS = "not equals"

    def is_not_equals(self, actual, expect):
        return actual != expect

    SAME = "same"

    def is_same(self, actual, expect):
        """
        Returns true if equal or if 2 lists, have the same items
        regardless of order.
        """
        ts = (list, tuple)
        same = actual == expect
        if not same and isinstance(actual, ts) and isinstance(expect, ts):
            same = bool(len(actual) and len(expect))
            same = same and all([item in expect for item in actual])
            same = same and all([item in actual for item in expect])
        return same

    # lists

    CONTAINS = "contains"

    def is_contains(self, actual, expect):
        return expect in actual

    NOT_CONTAINS = "not contains"

    def is_not_contains(self, actual, expect):
        return expect not in actual

    IN = "in"

    def is_in(self, actual, expect):
        return actual in expect

    NOT_IN = "not in"

    def is_not_in(self, actual, expect):
        return actual not in expect

    # greater/less than

    GREATER_THAN = "greater than"

    def is_greater_than(self, actual, expect):
        return actual > expect

    GT = "gt"

    def is_gt(self, actual, expect):
        return actual > expect

    GREATER_THAN_OR_EQUALS = "greater than or equals"

    def is_greater_than_or_equals(self, actual, expect):
        return actual >= expect

    GTE = "gte"

    def is_gte(self, actual, expect):
        return actual >= expect

    LESS_THAN = "less than"

    def is_less_than(self, actual, expect):
        return actual < expect

    LT = "lt"

    def is_lt(self, actual, expect):
        return actual < expect

    LESS_THAN_OR_EQUALS = "less than or equals"

    def is_less_than_or_equals(self, actual, expect):
        return actual <= expect

    LTE = "lte"

    def is_lte(self, actual, expect):
        return actual <= expect

    # type

    TYPE = "type"

    def is_type(self, actual, expect):
        return isinstance(actual, locate(expect))

    # regex

    REGEX = "regex"

    def is_regex(self, actual, expect):
        return bool(search(expect, actual))

    # evaluate

    def evaluate(self, actual, expected):
        method = getattr(self, "is_%s" % self.value.replace(" ", "_"))
        try:
            return method(actual, expected)
        except:
            return False



from rigor.reporting.cucumber import Cucumber
import yaml
import related
import os
import json


ROOT_DIR = os.path.join(os.path.dirname(__file__), "files")


def get_cuke(filename):
    fp = open(os.path.join(ROOT_DIR, filename))
    original = json.dumps(json.loads(fp.read()), indent=4, sort_keys=True)
    return related.from_json(original, Cucumber), original


def test_roundtrip_feature_1_json():
    cuke, original = get_cuke("feature_1.json")
    assert len(cuke.features) == 1
    assert cuke.features[0].uri == "features/one_passing_one_failing.feature"
    assert len(cuke.features[0].tags) == 1
    assert cuke.features[0].tags[0].name == "@a"

    generated = related.to_json(cuke.features)
    assert original == generated



import asyncio
import aiohttp

from . import SuiteResult, Runner


def execute(suite):
    loop = asyncio.get_event_loop()
    future = asyncio.ensure_future(do_suite(loop, suite))
    loop.run_until_complete(future)

    passed = []
    failed = []

    for result in future.result():
        # todo: handle exceptions...
        sink = passed if result.success else failed
        sink.append(result)

    success = bool(passed and not failed)

    return SuiteResult(
        suite=suite,
        passed=passed,
        failed=failed,
        success=success,
    )

async def do_suite(loop, suite):
    tasks = []
    connector = aiohttp.TCPConnector(limit_per_host=suite.concurrency)

    with aiohttp.ClientSession(loop=loop, connector=connector) as session:
        for case in suite.queued.values():
            for scenario in case.scenarios:
                runner = Runner(session=session, suite=suite, case=case,
                                scenario=scenario)

                tasks.append(asyncio.ensure_future(runner.do_run()))

        results = await asyncio.gather(*tasks, return_exceptions=True)

    return results



from rigor import Suite


def test_execute():
    directories = ["/Users/ianmaurer/code/knowledge/qa/api/therapies"]
    suite = Suite(directories=directories, tags_excluded=["broken"])
    result = suite.execute()
    # assert result.success
    # assert len(result.passed) == 1



from rigor import Runner, Namespace, Validator


JSON_DICT = {
    "args": {},
    "headers": {
        "Accept": "*/*",
        "Connection": "close",
        "Host": "httpbin.org",
        "User-Agent": "curl/7.51.0"
    },
    "origin": "127.0.0.1",
    "url": "https://httpbin.org/get"
}

VALIDATION_KWARGS = [
    dict(expect="${response.url}", actual="https://httpbin.org/get"),
    # dict(expect="${response.args}", actual={}),
    dict(expect="${response.origin}", actual="127.0.0.1"),
    dict(expect="${response['headers.Accept']}", actual="*/*"),
    dict(expect="${response['headers.Connection']}", actual="close"),
    # dict(expect="${response.headers}", actual=JSON_DICT['headers']),
]


def test_jmespath_access():
    response = Namespace(JSON_DICT)
    assert response['url'] == "https://httpbin.org/get"
    assert response.args == {}
    assert response["headers.Accept"] == "*/*"
    assert response.origin is not None
    assert response["headers.Connection"] == "close"


def test_mako_templates():
    response = Namespace(JSON_DICT)
    runner = Runner(response=response)

    for kwargs in VALIDATION_KWARGS:
        validator = Validator(**kwargs)
        result = runner.check_validation(validator)
        assert result.success, "%s != %s" % (result.expect, result.actual)



from rigor import Comparison

# equality


def test_equals():
    assert Comparison.EQUALS.evaluate(1, 1)
    assert Comparison.EQUALS.evaluate(None, None)
    assert Comparison.EQUALS.evaluate('hi', 'hi')
    assert not Comparison.EQUALS.evaluate('1', 1)
    assert not Comparison.EQUALS.evaluate('1', None)


def test_not_equals():
    assert not Comparison.NOT_EQUALS.evaluate(1, 1)
    assert not Comparison.NOT_EQUALS.evaluate(None, None)
    assert not Comparison.NOT_EQUALS.evaluate('hi', 'hi')
    assert Comparison.NOT_EQUALS.evaluate('1', 1)
    assert Comparison.NOT_EQUALS.evaluate('1', None)


def test_same():
    assert Comparison.SAME.evaluate((1, 3), (1, 3))
    assert Comparison.SAME.evaluate((3, 1), (1, 3))
    assert Comparison.SAME.evaluate((1, 3), (3, 1))
    assert Comparison.SAME.evaluate(1, 1)
    assert Comparison.SAME.evaluate(None, None)
    assert Comparison.SAME.evaluate('hi', 'hi')
    assert not Comparison.SAME.evaluate('1', 1)
    assert not Comparison.SAME.evaluate('1', None)


def test_contains():
    assert Comparison.CONTAINS.evaluate(range(3), 1)
    assert Comparison.CONTAINS.evaluate(('hi',), 'hi')
    assert not Comparison.CONTAINS.evaluate(range(3), '1')
    assert not Comparison.CONTAINS.evaluate(None, '1')
    assert not Comparison.CONTAINS.evaluate(None, None)


def test_not_contains():
    assert not Comparison.NOT_CONTAINS.evaluate(range(3), 1)
    assert not Comparison.NOT_CONTAINS.evaluate(('hi',), 'hi')
    assert Comparison.NOT_CONTAINS.evaluate(range(3), '1')
    assert not Comparison.NOT_CONTAINS.evaluate(None, '1')
    assert not Comparison.NOT_CONTAINS.evaluate(None, None)


def test_in():
    assert Comparison.IN.evaluate(1, range(3))
    assert Comparison.IN.evaluate('hi', ('hi',))
    assert not Comparison.IN.evaluate('1', range(3))
    assert not Comparison.IN.evaluate('1', None)
    assert not Comparison.IN.evaluate(None, None)


def test_not_in():
    assert not Comparison.NOT_IN.evaluate(1, range(3))
    assert not Comparison.NOT_IN.evaluate('hi', ('hi',))
    assert Comparison.NOT_IN.evaluate('1', range(3))
    assert not Comparison.NOT_IN.evaluate('1', None)
    assert not Comparison.NOT_IN.evaluate(None, None)


def test_greater_than():
    assert Comparison.GREATER_THAN.evaluate(1, 0)
    assert not Comparison.GREATER_THAN.evaluate(0, 0)
    assert not Comparison.GREATER_THAN.evaluate(0, 1)

    assert Comparison.GT.evaluate(1, 0)
    assert not Comparison.GT.evaluate(0, 0)
    assert not Comparison.GT.evaluate(0, 1)
    assert not Comparison.GT.evaluate(0, None)
    assert not Comparison.GT.evaluate(None, 0)
    assert not Comparison.GT.evaluate("1", 0)
    assert not Comparison.GT.evaluate(1, "0")
    assert not Comparison.GT.evaluate(0, "1")


def test_greater_than_or_equals():
    assert Comparison.GREATER_THAN_OR_EQUALS.evaluate(1, 1)
    assert Comparison.GREATER_THAN_OR_EQUALS.evaluate(2, 1)
    assert Comparison.GREATER_THAN_OR_EQUALS.evaluate(1, -1)
    assert not Comparison.GREATER_THAN_OR_EQUALS.evaluate(1, 3)

    assert Comparison.GTE.evaluate(1, 1)
    assert Comparison.GTE.evaluate(2, 1)
    assert Comparison.GTE.evaluate(1, -1)
    assert not Comparison.GTE.evaluate(1, 3)


def test_less_than_or_equals():
    assert Comparison.LESS_THAN_OR_EQUALS.evaluate(1, 1)
    assert Comparison.LESS_THAN_OR_EQUALS.evaluate(1, 2)
    assert Comparison.LESS_THAN_OR_EQUALS.evaluate(-1, 1)
    assert not Comparison.LESS_THAN_OR_EQUALS.evaluate(3, 1)

    assert Comparison.LTE.evaluate(1, 1)
    assert Comparison.LTE.evaluate(1, 2)
    assert Comparison.LTE.evaluate(-1, 1)
    assert not Comparison.LTE.evaluate(3, 1)


def test_type():
    assert Comparison.TYPE.evaluate(1, "int")
    assert Comparison.TYPE.evaluate(2.0, "float")
    assert Comparison.TYPE.evaluate("hello", "str")
    assert Comparison.TYPE.evaluate([1, 2], "list")
    assert Comparison.TYPE.evaluate(dict(a=1), "dict")


def test_regex():
    assert Comparison.REGEX.evaluate("", ".*")
    assert Comparison.REGEX.evaluate("2222", "\d+")
    assert Comparison.REGEX.evaluate("111-33-3333", "\d{3}-\d{2}-\d{4}")
    assert Comparison.REGEX.evaluate("John Q. Public", "\w+ \w\. \w+")



import json
import os
from itertools import product

import related

from . import Comparison, Method, Namespace


class Iterator(Namespace):

    def iterate(self, namespace):
        if len(self):
            # determine method (product or zip)
            d = self.copy()
            method_key = d.pop("__method__", "zip")
            method = dict(zip=zip, product=product).get(method_key, zip)

            # evaluate values in namespace
            values = [self.render(value, namespace)
                      for value in d.values()]

            # *values => assumes all values are iterable and thus can zipped.
            for zipped_values in method(*values):

                # combine zipped values and construct a Namespace object
                yield Namespace(dict(zip(d.keys(), zipped_values)))
        else:
            yield Namespace()


@related.immutable
class Validator(object):
    actual = related.ChildField(object)
    expect = related.ChildField(object)
    compare = related.ChildField(Comparison, default=Comparison.EQUALS)


@related.immutable
class Requestor(object):
    path = related.StringField()
    method = related.ChildField(Method, default=Method.GET)
    domain = related.StringField(required=False)
    headers = related.ChildField(Namespace, required=False)
    params = related.ChildField(Namespace, required=False)
    data = related.ChildField(object, required=False)
    status = related.SequenceField(int, required=False)

    def get_params(self, namespace):
        dd = self.params.evaluate(namespace) if self.params else {}

        params = []
        for key, value in dd.items():
            if isinstance(value, (tuple, list, set)):
                for item in value:
                    params.append((key, str(item)))
            else:
                params.append((key, str(value)))

        return params

    def get_data(self, namespace):
        # flatten data to a string that will include ${expressions} to render
        ds = self.data if isinstance(self.data, str) else json.dumps(self.data)

        # render data string (ds) template
        rendered = Namespace.render(ds, namespace)

        # dump rendered value to a json string, if not already a string
        return rendered if isinstance(rendered, str) else json.dumps(rendered)


@related.immutable
class Step(object):
    description = related.StringField()
    request = related.ChildField(Requestor)
    extract = related.ChildField(Namespace, default=Namespace())
    iterate = related.ChildField(Iterator, default=Iterator())
    validate = related.SequenceField(Validator, required=False)


@related.immutable
class Case(object):
    name = related.StringField(required=False, default=None)
    steps = related.SequenceField(Step, default=[])
    scenarios = related.SequenceField(Namespace, default=[Namespace()])
    format = related.StringField(default="1.0")
    domain = related.StringField(required=False)
    tags = related.SequenceField(str, required=False)
    headers = related.ChildField(Namespace, required=False)
    file_path = related.StringField(default=None)
    is_valid = related.BooleanField(default=True)

    @classmethod
    def load(cls, file_path):
        return related.from_yaml(open(file_path), Case, file_path=file_path)

    @classmethod
    def loads(cls, content, file_path=None):
        try:
            return related.from_yaml(content, Case, file_path=file_path,
                                     object_pairs_hook=dict)
        except Exception as exc:
            print("Load Failed: %s: %s" % (file_path, exc))  # todo: logging
            return Case(file_path=file_path, is_valid=False)

    def is_active(self, included, excluded):
        has_steps = len(self.steps) > 0
        is_included = not included or overlap(included, self.tags)
        is_excluded = excluded and overlap(excluded, self.tags)
        return self.is_valid and has_steps and is_included and not is_excluded

    @property
    def dir_path(self):
        return os.path.dirname(self.file_path)


@related.mutable
class Suite(object):
    # cli options
    domain = related.StringField(default="http://localhost:8000")
    directories = related.SequenceField(str, default=None)
    file_prefixes = related.SequenceField(str, default=None)
    extensions = related.SequenceField(str, default=["rigor"])
    tags_included = related.SequenceField(str, default=None)
    tags_excluded = related.SequenceField(str, default=None)
    concurrency = related.IntegerField(default=20)

    # collect
    queued = related.MappingField(Case, "file_path", default={})
    skipped = related.MappingField(Case, "file_path", default={})

    def __attrs_post_init__(self):
        from . import collect
        collect(self)

    def get_case(self, path, filename=None):
        file_path = os.path.join(path, filename) if filename else path
        return self.queued.get(file_path) or self.skipped.get(file_path)

    def add_case(self, case):
        if case.is_active(self.tags_included, self.tags_excluded):
            self.queued.add(case)
        else:
            self.skipped.add(case)

    def execute(self):
        from . import execute
        return execute(self)


# utilities

def overlap(l1, l2):
    return set(l1 or []) & set(l2 or [])



import related
import jmespath
import ast
import re

from mako.template import Template


class Namespace(related.ImmutableDict):

    def __getattr__(self, item):
        return self.__getitem__(item)

    def __getitem__(self, name):
        return self.get(name) if name in self else jmespath.search(name, self)

    def evaluate(self, namespace, existing=None):
        values = existing or {}

        for key, value in self.items():
            values[key] = self.render(value, namespace)

        return Namespace(values)

    @classmethod
    def wrap(cls, value):
        if isinstance(value, dict) and not isinstance(value, cls):
            return cls(value)

        if isinstance(value, list):
            new_list = []
            for item in value:
                new_list.append(cls.wrap(item))
            value = new_list

        return value

    @classmethod
    def render(cls, value, namespace):
        if isinstance(value, str):
            template = Template(value)

            try:
                rendered = template.render(**namespace)
            except:
                raise

            try:
                # eval if rendered value is list, dict, int or float
                is_list_or_dict = rendered.strip()[0] in ("{", "[")
                is_number = re.match("^[\d\.]+$", rendered.strip())
                assert is_list_or_dict or is_number

                value = ast.literal_eval(rendered)

            except:
                value = rendered

        value = cls.wrap(value)

        return value



import asyncio
import time

import aiohttp
import related

from . import Case, Namespace, Step, Suite, Validator, Functions

# https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#2xx_Success
HTTP_SUCCESS = [200, 201, 202, 203, 204, 205, 206, 207, 208, 226]


@related.immutable
class ValidationResult(object):
    actual = related.ChildField(object)
    expect = related.ChildField(object)
    success = related.BooleanField()


@related.immutable
class Fetch(object):
    url = related.StringField()
    method = related.StringField()
    kwargs = related.ChildField(dict)


@related.immutable
class StepResult(object):
    step = related.ChildField(Step)
    success = related.BooleanField()
    fetch = related.ChildField(Fetch, required=False)
    response = related.ChildField(Namespace, required=False)
    status = related.IntegerField(required=False)
    validations = related.SequenceField(ValidationResult, required=False)


@related.immutable
class ScenarioResult(object):
    suite = related.ChildField(Suite)
    case = related.ChildField(Case)
    scenario = related.ChildField(Namespace)
    success = related.BooleanField()
    uuid = related.UUIDField()
    step_results = related.SequenceField(StepResult, default=[])
    running_time = related.FloatField(required=False)


@related.immutable
class SuiteResult(object):
    success = related.BooleanField()
    suite = related.ChildField(Suite)
    passed = related.SequenceField(ScenarioResult, default=[])
    failed = related.SequenceField(ScenarioResult, default=[])


@related.mutable
class Runner(object):
    uuid = related.UUIDField()
    session = related.ChildField(aiohttp.ClientSession, required=False)
    suite = related.ChildField(Suite, required=False)
    case = related.ChildField(Case, required=False)

    scenario = related.ChildField(Namespace, default=Namespace())
    extract = related.ChildField(Namespace, default=Namespace())
    iterate = related.ChildField(Namespace, default=Namespace())
    response = related.ChildField(Namespace, default=Namespace())

    def __attrs_post_init__(self):
        self.scenario = self.scenario.evaluate(self.namespace)

    @property
    def namespace(self):
        # make iterate namespace top-level
        values = self.iterate.copy() if self.iterate else {}

        # make extract namespace top-level accessors (overrides iterate!)
        values.update(self.extract if self.extract else {})

        # add handles to namespaces (overrides extract and iterate!)
        values.update(dict(__uuid__=self.uuid,
                           scenario=self.scenario,
                           extract=self.extract,
                           response=self.response,
                           iterate=self.iterate))

        # add state-aware functions such as list_yaml (overrides all before!)
        values.update(Functions(self).function_map())

        return values

    async def do_run(self):
        success = True
        start_time = time.time()
        step_results = []

        # iterate steps
        async for step_result in self.iter_steps():
            step_results.append(step_result)
            success = step_result.success
            if not success:
                break

        running_time = time.time() - start_time

        return ScenarioResult(
            uuid=self.uuid,
            suite=self.suite,
            case=self.case,
            scenario=self.scenario,
            success=success,
            step_results=step_results,
            running_time=running_time,
        )

    async def iter_steps(self):
        for step in self.case.steps:
            await asyncio.sleep(0.5)  # todo: replace...

            for self.iterate in step.iterate.iterate(self.namespace):
                # create and do fetch
                fetch = self.create_fetch(step.request)
                self.response, status = await self.do_fetch(fetch)

                # extract response
                self.extract = self.do_extract(step)

                # validate response
                validations, success = self.do_validate(step, status)

                # add step result
                yield StepResult(step=step, fetch=fetch,
                                 response=self.response, status=status,
                                 validations=validations, success=success)

    def create_fetch(self, request):
        namespace = self.namespace

        # construct url
        domain = request.domain or self.case.domain or self.suite.domain
        path = Namespace.render(request.path, namespace)
        url = "%s/%s" % (domain, path)

        # construct method
        method = request.method.value.lower()

        # headers
        headers = {"content-type": "application/json"}
        headers.update(related.to_dict(self.case.headers) or {})
        headers.update(related.to_dict(request.headers) or {})
        headers = Namespace(headers)

        # kwargs
        kwargs = dict(headers=headers, timeout=None,
                      data=request.get_data(namespace),
                      params=request.get_params(namespace))

        return Fetch(method=method, url=url, kwargs=kwargs)

    async def do_fetch(self, fetch):
        # make request and store response
        async with self.session.request(fetch.method, fetch.url,
                                        **fetch.kwargs) as context:
            try:
                response = Namespace(await context.json())
            except:
                response = Namespace()

            status = context.status
            return response, status

    def do_extract(self, step):
        existing = related.to_dict(self.extract) if self.extract else {}
        return step.extract.evaluate(self.namespace, existing)

    def do_validate(self, step, status):
        results = []

        # status check
        expected_status = step.request.status or HTTP_SUCCESS
        results.append(ValidationResult(actual=status,
                                        expect=expected_status,
                                        success=status in expected_status))

        # validators check
        for validator in step.validate or []:
            results.append(self.check_validation(validator))

        # determine success and return
        success = all([result.success for result in results])
        return results, success

    def check_validation(self, validator):
        actual = Namespace.render(validator.actual, self.namespace)
        expect = Namespace.render(validator.expect, self.namespace)
        success = validator.compare.evaluate(actual, expect)
        return ValidationResult(actual=actual,
                                expect=expect, success=success)



from rigor import Suite, Method, Namespace, Validator, Runner
from collections import OrderedDict

import pytest
import os
import related
import json

ROOT_DIR = os.path.join(os.path.dirname(__file__), "httpbin")


@pytest.fixture
def suite():
    directories = [ROOT_DIR]
    return Suite(directories=directories, tags_excluded=["broken"])


def test_collect(suite):
    assert suite.tags_excluded == ["broken"]
    assert len(suite.skipped) == 2
    assert len(suite.queued) == 8


def test_execute(suite):
    result = suite.execute()
    assert result.success
    assert len(result.passed) == 8


def test_case_get(suite):
    case = suite.get_case(ROOT_DIR, "get.rigor")

    # check case root
    assert case.name == "Get"
    assert case.format == "1.0"
    assert case.domain == "https://httpbin.org"
    assert case.tags == ["working"]
    assert len(case.steps) == 1

    # check case steps
    step = case.steps[0]
    assert step.description == "Get call with no parameters"
    assert step.request.path == "/get"
    assert step.validate == [
        Validator(expect="${response.url}", actual="https://httpbin.org/get"),
        Validator(expect="${response.args}", actual=OrderedDict()),
        Validator(expect="${response['headers.Accept']}", actual="*/*"),
        Validator(expect="${response['headers.Connection']}", actual="close"),
    ]


def test_case_params(suite):
    case = suite.get_case(ROOT_DIR, "params.rigor")

    # check case root
    assert case.name == "Params"
    assert case.format == "1.0"
    assert case.domain == "https://httpbin.org"
    assert case.tags == ["working"]
    assert len(case.steps) == 1
    assert len(case.scenarios) == 1

    # check scenarios
    assert case.scenarios[0] == Namespace(value=1)


def test_case_http_status(suite):
    case = suite.get_case(ROOT_DIR, "http_status.rigor")

    # check case root
    assert case.name == "HTTP Status"
    assert case.format == "1.0"
    assert case.domain == "https://httpbin.org"
    assert len(case.steps) == 4
    assert len(case.scenarios) == 1  # default empty scenario

    # check step
    assert case.steps[0].request.status == []
    assert case.steps[1].request.status == [200]
    assert case.steps[2].request.status == [404]
    assert case.steps[3].request.status == [418]


def test_case_iterate(suite):
    case = suite.get_case(ROOT_DIR, "iterate.rigor")
    assert len(case.steps) == 4
    step = case.steps[0]
    namespace = Runner(case=case).namespace

    assert list(step.iterate.iterate(namespace)) == [
        dict(show_env=0, other="A"),
        dict(show_env=1, other="B"),
    ]
    step = case.steps[2]
    assert list(step.iterate.iterate(namespace)) == [
        dict(show_env=0, other="A"),
        dict(show_env=0, other="B"),
        dict(show_env=0, other="C"),
        dict(show_env=0, other="D"),
        dict(show_env=0, other="E"),
        dict(show_env=0, other="F"),
        dict(show_env=1, other="A"),
        dict(show_env=1, other="B"),
        dict(show_env=1, other="C"),
        dict(show_env=1, other="D"),
        dict(show_env=1, other="E"),
        dict(show_env=1, other="F"),
    ]


def test_case_list_yaml(suite):
    case = suite.get_case(ROOT_DIR, "list_yaml.rigor")
    assert len(case.steps) == 1
    step = case.steps[0]
    assert step.iterate == dict(data="${list_yaml('./data/example.yaml')}")

    namespace = Runner(case=case).namespace
    iterate = list(step.iterate.iterate(namespace))

    iter0 = iterate[0]
    assert isinstance(iter0, dict)
    assert iter0['data']['request']['query'] == "field:val"


def test_case_load_yaml(suite):
    case = suite.get_case(ROOT_DIR, "load_yaml.rigor")
    assert len(case.scenarios) == 1
    assert len(case.steps) == 1

    scenario = case.scenarios[0]
    assert scenario == dict(data="${load_yaml('./data/example.yaml')}")



import click
import jmespath

from . import Suite
import related
import json
import sys


@click.command()
@click.argument('directories', nargs=-1)
@click.option('--domain', default="http://localhost:8000",
              help="Domain name (e.g. http://localhost:8000)")
@click.option('--include', '-i', multiple=True,
              help="Include tag of cases. (e.g. smoke)")
@click.option('--exclude', '-e', multiple=True,
              help="Exclude tag of cases to run. (e.g. broken)")
@click.option('--prefix', '-p', multiple=True,
              help="Filter cases by file prefix. (e.g. smoke_)")
@click.option('--extensions', '-e', multiple=True,
              help="Filter cases by file extension. (e.g. rigor)")
@click.option('--concurrency', '-c', type=int, default=20,
              help='# of concurrent HTTP requests. (default: 20)')
def main(directories, domain, include, exclude, prefix, extensions,
         concurrency):
    # remove preceding . if provided in extension (.rigor => rigor)
    extensions = [ext[1:] if ext.startswith(".") else ext
                  for ext in extensions or []]

    # collect
    suite = Suite(directories=directories, domain=domain,
                  tags_included=include, tags_excluded=exclude,
                  file_prefixes=prefix, extensions=extensions,
                  concurrency=concurrency)

    # execute
    results = suite.execute()

    # k = 0
    # j = 1
    # i = 0
    # for failure in results.failed:
    #     print("Failure #%s" % i)
    #     print("Case:\n%s (%s)\n" % (failure.case.name, failure.case.file_path))
    #     print("Step:\n%s\n" % related.to_json(failure.fail_step))
    #     print("Scenario:\n%s\n" % related.to_json(failure.scenario))
    #     print("Last Fetch:\n%s\n" % related.to_json(failure.fetch))
    #     print("Last Response:\n%s\n" % related.to_json(failure.response))
    #     print("Failed Validations:")
    #     for validation in failure.fail_validations:
    #         print(related.to_json(validation))
    #     print("\n")
    #     i += 1

    print("Passed: %s" % len(results.passed))
    print("Failed: %s" % len(results.failed))

    # report success
    return 1 if results.failed else 0


if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-

from .collect import (
    collect
)

from .enums import (
    Comparison,
    Method,
    Status,
)

from .namespace import (
    Namespace,
)

from .functions import (
    Functions
)

from .model import (
    Case,
    Step,
    Suite,
    Validator,
)

from .state import (
    Runner,
    SuiteResult,
)

from .execute import (
    execute
)

from .cli import (
    main
)


__all__ = [
    # collect.py
    "collect",

    # enums.py
    "Comparison",
    "Method",
    "Status",

    # execute.py
    "execute",

    # functions.py
    "Functions",

    # namespace.py
    "Namespace",

    # model.py
    "Case",
    "Step",
    "Suite",
    "Validator",

    # state.py
    "Runner",
    "SuiteResult",

    # cli.py
    "main",
]


__author__ = """Ian Maurer"""
__email__ = 'ian@genomoncology.com'
__version__ = '0.0.13'

__uri__ = "http://www.github.com/genomoncology/rigor"
__copyright__ = "Copyright (c) 2017 genomoncology.com"
__description__ = "Rigor: No-Nonsense Asynchronous REST API Testing"
__doc__ = __description__ + " <" + __uri__ + ">"
__license__ = "MIT"
__title__ = "rigor"



import asyncio
import aiofiles
import glob
import os


def collect(suite):
    loop = asyncio.get_event_loop()
    future = asyncio.ensure_future(async_collect(suite))
    loop.run_until_complete(future)

    for case in future.result():
        suite.add_case(case)


async def async_collect(suite):
    tasks = []
    for pattern in glob_patterns(suite):
        for file_path in glob.iglob(pattern, recursive=True):
            tasks.append(asyncio.ensure_future(collect_case(suite, file_path)))
    cases = await asyncio.gather(*tasks)
    return cases


async def collect_case(suite, file_path):
    from . import Case
    async with aiofiles.open(file_path, mode='r') as f:
        return Case.loads(await f.read(), file_path=file_path)


def glob_patterns(suite):
    for directory in suite.directories or ['.']:
        if os.path.exists(directory) and os.path.isfile(directory):
            yield directory

        for file_prefix in suite.file_prefixes or ['']:
            for extension in suite.extensions or ['rigor']:
                file_pattern = "%s*.%s" % (file_prefix, extension)
                yield os.path.join(directory, "**", file_pattern)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.md') as history_file:
    history = history_file.read()

with open('dev-requirements.txt') as dev_requirements_file:
    tests_require = [r.strip() for r in dev_requirements_file.readlines()]

setup(
    name="rigor",
    version='0.0.13',

    package_dir={
        '': 'src'
    },

    packages=[
        "rigor",
    ],

    include_package_data=True,

    install_requires=[
        "related >= 0.4",
        "aiohttp==2.1.0",
        "aiofiles==0.3.1",
        "jmespath==0.9.3",
        "Mako==1.0.6",
        "click==6.7",
    ],

    setup_requires=[
        'pytest-runner',
    ],

    tests_require=tests_require,

    license="MIT license",

    keywords='',
    description="rigor",
    long_description="%s\n\n%s" % (readme, history),

    entry_points={
        'console_scripts': [
            'rigor=rigor.cli:main',
        ]
    },

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



import os
import glob
import yaml


class Functions(object):

    def __init__(self, runner):
        self.runner = runner

    def get_pattern(self, relative_pattern):
        return os.path.join(self.runner.case.dir_path, relative_pattern)

    def load_yaml(self, filename):
        return next(self.iter_yaml(filename))

    def iter_yaml(self, *filenames):
        for filename in filenames:
            file_pattern = self.get_pattern(filename)
            for path in glob.glob(file_pattern, recursive=True):
                content = self.read_render(path)
                for doc in yaml.load_all(content):
                    yield doc

    def list_yaml(self, *filenames):
        return list(self.iter_yaml(*filenames))

    def function_map(self):
        return dict(list_yaml=self.list_yaml, load_yaml=self.load_yaml)

    def read_render(self, filepath):
        from . import Namespace
        content = open(filepath).read()
        return Namespace.render(content, self.runner.namespace)
